@@ -15,18 +15,20 @@
 
 import abc
 
-from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import resource_helper
-from neutron.common import exceptions as nexception
-from neutron.plugins.common import constants as p_const
 from neutron.services import service_base
+from neutron_lib.api import converters
+from neutron_lib.api import extensions
+from neutron_lib.api import validators
 from neutron_lib import constants
+from neutron_lib.db import constants as db_const
+from neutron_lib import exceptions as nexception
 from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
 from neutron_fwaas._i18n import _
+from neutron_fwaas.common import fwaas_constants
 
 
 LOG = logging.getLogger(__name__)
@@ -107,7 +109,7 @@ class FirewallRuleInUse(nexception.InUse):
 
 class FirewallRuleNotAssociatedWithPolicy(nexception.InvalidInput):
     message = _("Firewall Rule %(firewall_rule_id)s is not associated "
-                " with Firewall Policy %(firewall_policy_id)s.")
+                "with Firewall Policy %(firewall_policy_id)s.")
 
 
 class FirewallRuleInvalidProtocol(nexception.InvalidInput):
@@ -142,15 +144,14 @@ class FirewallRuleInfoMissing(nexception.InvalidInput):
 class FirewallIpAddressConflict(nexception.InvalidInput):
     message = _("Invalid input - IP addresses do not agree with IP Version")
 
-# TODO(dougwig) - once this exception is out of neutron, restore this
-#class FirewallInternalDriverError(nexception.NeutronException):
-#    """Fwaas exception for all driver errors.
-#
-#    On any failure or exception in the driver, driver should log it and
-#    raise this exception to the agent
-#    """
-#    message = _("%(driver)s: Internal driver error.")
-FirewallInternalDriverError = nexception.FirewallInternalDriverError
+
+class FirewallInternalDriverError(nexception.NeutronException):
+    """Fwaas exception for all driver errors.
+
+    On any failure or exception in the driver, driver should log it and
+    raise this exception to the agent
+    """
+    message = _("%(driver)s: Internal driver error.")
 
 
 class FirewallRuleConflict(nexception.Conflict):
@@ -175,20 +176,19 @@ fw_valid_action_values = [FWAAS_ALLOW, FWAAS_DENY, FWAAS_REJECT]
 def convert_protocol(value):
     if value is None:
         return
-    if value.isdigit():
+    if (isinstance(value, six.integer_types) or
+       (isinstance(value, six.string_types) and value.isdigit())):
         val = int(value)
         if 0 <= val <= 255:
             return val
         else:
             raise FirewallRuleInvalidProtocol(
-                protocol=value,
-                values=fw_valid_protocol_values)
-    elif value.lower() in fw_valid_protocol_values:
-        return value.lower()
-    else:
-        raise FirewallRuleInvalidProtocol(
-            protocol=value,
-            values=fw_valid_protocol_values)
+                protocol=value, values=fw_valid_protocol_values)
+    elif isinstance(value, six.string_types):
+        if value.lower() in fw_valid_protocol_values:
+            return value.lower()
+    raise FirewallRuleInvalidProtocol(
+        protocol=value, values=fw_valid_protocol_values)
 
 
 def convert_action_to_case_insensitive(value):
@@ -226,18 +226,19 @@ def _validate_port_range(data, key_specs=None):
 def _validate_ip_or_subnet_or_none(data, valid_values=None):
     if data is None:
         return None
-    msg_ip = attr._validate_ip_address(data, valid_values)
+    msg_ip = validators.validate_ip_address(data, valid_values)
     if not msg_ip:
         return
-    msg_subnet = attr._validate_subnet(data, valid_values)
+    msg_subnet = validators.validate_subnet(data, valid_values)
     if not msg_subnet:
         return
     return _("%(msg_ip)s and %(msg_subnet)s") % {'msg_ip': msg_ip,
                                                  'msg_subnet': msg_subnet}
 
 
-attr.validators['type:port_range'] = _validate_port_range
-attr.validators['type:ip_or_subnet_or_none'] = _validate_ip_or_subnet_or_none
+validators.validators['type:port_range'] = _validate_port_range
+validators.validators['type:ip_or_subnet_or_none'] = \
+    _validate_ip_or_subnet_or_none
 
 
 RESOURCE_ATTRIBUTE_MAP = {
@@ -249,17 +250,18 @@ RESOURCE_ATTRIBUTE_MAP = {
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'firewall_policy_id': {'allow_post': False, 'allow_put': False,
                                'validate': {'type:uuid_or_none': None},
                                'is_visible': True},
         'shared': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
+                   'default': False,
+                   'convert_to': converters.convert_to_boolean,
                    'is_visible': True, 'required_by_policy': True,
                    'enforce_policy': True},
         'protocol': {'allow_post': True, 'allow_put': True,
@@ -267,7 +269,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                      'convert_to': convert_protocol,
                      'validate': {'type:values': fw_valid_protocol_values}},
         'ip_version': {'allow_post': True, 'allow_put': True,
-                       'default': 4, 'convert_to': attr.convert_to_int,
+                       'default': 4, 'convert_to': converters.convert_to_int,
                        'validate': {'type:values': [4, 6]},
                        'is_visible': True},
         'source_ip_address': {'allow_post': True, 'allow_put': True,
@@ -292,8 +294,8 @@ RESOURCE_ATTRIBUTE_MAP = {
                    'validate': {'type:values': fw_valid_action_values},
                    'is_visible': True, 'default': 'deny'},
         'enabled': {'allow_post': True, 'allow_put': True,
-                    'default': True, 'convert_to': attr.convert_to_boolean,
-                    'is_visible': True},
+                    'default': True, 'is_visible': True,
+                    'convert_to': converters.convert_to_boolean},
     },
     'firewall_policies': {
         'id': {'allow_post': False, 'allow_put': False,
@@ -304,23 +306,23 @@ RESOURCE_ATTRIBUTE_MAP = {
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'shared': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
-                   'is_visible': True, 'required_by_policy': True,
-                   'enforce_policy': True},
+                   'default': False, 'enforce_policy': True,
+                   'convert_to': converters.convert_to_boolean,
+                   'is_visible': True, 'required_by_policy': True},
         'firewall_rules': {'allow_post': True, 'allow_put': True,
                            'validate': {'type:uuid_list': None},
-                           'convert_to': attr.convert_none_to_empty_list,
+                           'convert_to': converters.convert_none_to_empty_list,
                            'default': None, 'is_visible': True},
         'audited': {'allow_post': True, 'allow_put': True,
-                    'default': False, 'convert_to': attr.convert_to_boolean,
-                    'is_visible': True},
+                    'default': False, 'is_visible': True,
+                    'convert_to': converters.convert_to_boolean},
     },
     'firewalls': {
         'id': {'allow_post': False, 'allow_put': False,
@@ -331,22 +333,21 @@ RESOURCE_ATTRIBUTE_MAP = {
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
                         'validate': {'type:string':
-                                     attr.DESCRIPTION_MAX_LEN},
+                                     db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'admin_state_up': {'allow_post': True, 'allow_put': True,
-                           'default': True,
-                           'convert_to': attr.convert_to_boolean,
-                           'is_visible': True},
+                           'default': True, 'is_visible': True,
+                           'convert_to': converters.convert_to_boolean},
         'status': {'allow_post': False, 'allow_put': False,
                    'is_visible': True},
         'shared': {'allow_post': True, 'allow_put': True,
-                   'default': False, 'convert_to': attr.convert_to_boolean,
-                   'is_visible': False, 'required_by_policy': True,
-                   'enforce_policy': True},
+                   'default': False, 'enforce_policy': True,
+                   'convert_to': converters.convert_to_boolean,
+                   'is_visible': False, 'required_by_policy': True},
         'firewall_policy_id': {'allow_post': True, 'allow_put': True,
                                'validate': {'type:uuid_or_none': None},
                                'is_visible': True},
@@ -402,12 +403,11 @@ class Firewall(extensions.ExtensionDescriptor):
         special_mappings = {'firewall_policies': 'firewall_policy'}
         plural_mappings = resource_helper.build_plural_mappings(
             special_mappings, RESOURCE_ATTRIBUTE_MAP)
-        attr.PLURALS.update(plural_mappings)
         action_map = {'firewall_policy': {'insert_rule': 'PUT',
                                           'remove_rule': 'PUT'}}
         return resource_helper.build_resource_info(plural_mappings,
                                                    RESOURCE_ATTRIBUTE_MAP,
-                                                   p_const.FIREWALL,
+                                                   fwaas_constants.FIREWALL,
                                                    action_map=action_map,
                                                    register_quota=True)
 
@@ -430,10 +430,10 @@ class Firewall(extensions.ExtensionDescriptor):
 class FirewallPluginBase(service_base.ServicePluginBase):
 
     def get_plugin_name(self):
-        return p_const.FIREWALL
+        return fwaas_constants.FIREWALL
 
     def get_plugin_type(self):
-        return p_const.FIREWALL
+        return fwaas_constants.FIREWALL
 
     def get_plugin_description(self):
         return 'Firewall service plugin'
