@@ -15,9 +15,7 @@
 
 import copy
 import mock
-from oslo_config import cfg
 
-from neutron.agent.common import config as a_cfg
 from neutron.tests import base
 from neutron.tests.unit.api.v2 import test_base as test_api_v2
 import neutron_fwaas.services.firewall.drivers.linux.iptables_fwaas as fwaas
@@ -36,7 +34,6 @@ FW_LEGACY = 'legacy'
 class IptablesFwaasTestCase(base.BaseTestCase):
     def setUp(self):
         super(IptablesFwaasTestCase, self).setUp()
-        cfg.CONF.register_opts(a_cfg.ROOT_HELPER_OPTS, 'AGENT')
         self.utils_exec_p = mock.patch(
             'neutron.agent.linux.utils.execute')
         self.utils_exec = self.utils_exec_p.start()
@@ -144,9 +141,9 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         func(distributed_mode, apply_list, firewall)
         invalid_rule = '-m state --state INVALID -j DROP'
         est_rule = '-m state --state ESTABLISHED,RELATED -j ACCEPT'
-        rule1 = '-p tcp --dport 80  -s 10.24.4.2  -j ACCEPT'
-        rule2 = '-p tcp --dport 22    -j DROP'
-        rule3 = '-p tcp --dport 23    -j REJECT'
+        rule1 = '-p tcp -m tcp --dport 80 -s 10.24.4.2 -j ACCEPT'
+        rule2 = '-p tcp -m tcp --dport 22 -j DROP'
+        rule3 = '-p tcp -m tcp --dport 23 -j REJECT'
         ingress_chain = 'iv4%s' % firewall['id']
         egress_chain = 'ov4%s' % firewall['id']
         bname = fwaas.iptables_manager.binary_name
